import * as i0 from '@angular/core';
import { EventEmitter, forwardRef, ElementRef, Component, ChangeDetectionStrategy, Input, Output, ViewChild, HostBinding, NgModule } from '@angular/core';
import { NG_VALUE_ACCESSOR } from '@angular/forms';
import { EditorView } from '@codemirror/view';
import { languages } from '@codemirror/language-data';
import { Annotation, Compartment, EditorState, StateEffect } from '@codemirror/state';
import { timer } from 'rxjs';
import { basicSetup, minimalSetup } from 'codemirror';
import { oneDark } from '@codemirror/theme-one-dark';
import { material } from '@uiw/codemirror-theme-material';
import { CommonModule } from '@angular/common';

const selectableLanguages = languages.map(lang => ({ name: lang.name, value: lang.name.toLowerCase() }))
    .sort((a, b) => a.name.localeCompare(b.name));
const External = Annotation.define();
class CodeMirrorComponent {
    constructor(elementRef, ngZone, renderer, _differs) {
        this.elementRef = elementRef;
        this.ngZone = ngZone;
        this.renderer = renderer;
        this._differs = _differs;
        this.value = null;
        this.onTouchedCallback = () => { };
        this.onChangeCallback = () => { };
        this.autoMaxHeight = 0;
        this.delayRefreshTime = 200;
        this.extensions = [];
        this.focusChange = new EventEmitter();
        this.codemirrorClassName = true;
        this._readonlyConf = new Compartment();
        this._themeConf = new Compartment();
        this._placeholderConf = new Compartment();
        this._indentWithTabConf = new Compartment();
        this._indentUnitConf = new Compartment();
        this._lineWrappingConf = new Compartment();
        this._highlightWhitespaceConf = new Compartment();
        this._languageConf = new Compartment();
        this._updateListener = EditorView.updateListener.of(vu => {
            if (vu.docChanged && !vu.transactions.some(tr => tr.annotation(External))) {
                const value = vu.state.doc.toString();
                this.onChangeCallback(value);
            }
        });
    }
    ngOnInit() {
        if (!this.languages) {
            this.languages = languages;
        }
        if (!this._differ && this.options) {
            this._differ = this._differs.find(this.options).create();
        }
        this.initializeCodemirror();
        timer(this.delayRefreshTime).subscribe(() => {
            if (this.autoMaxHeight) {
                this.applyEditorHeight();
            }
            this.view.requestMeasure();
        });
    }
    ngOnChanges(simpleChanges) {
        const optionsChange = simpleChanges.options;
        if (optionsChange && !optionsChange.firstChange) {
            const changes = this._differ.diff(this.options);
            if (changes) {
                changes.forEachChangedItem((option) => this.setOptionIfChanged(option.key, option.currentValue));
                changes.forEachAddedItem((option) => this.setOptionIfChanged(option.key, option.currentValue));
                changes.forEachRemovedItem((option) => this.setOptionIfChanged(option.key, option.currentValue));
            }
        }
    }
    registerOnChange(fn) {
        this.onChangeCallback = fn;
    }
    registerOnTouched(fn) {
        this.onTouchedCallback = fn;
    }
    writeValue(value) {
        const valid = value !== null && value !== undefined;
        if (this.view && valid && this.value !== value) {
            this.setValue(value);
        }
        this.value = value;
    }
    initializeCodemirror() {
        this.ngZone.runOutsideAngular(() => {
            if (!this.options) {
                throw new Error("options is required");
            }
            this.view = new EditorView({
                parent: this.elementRef.nativeElement,
                state: EditorState.create({ doc: this.value, extensions: this._getAllExtensions() }),
            });
            if (this.options.autofocus) {
                this.view.focus();
            }
            if (this.options.readonly) {
                this.setReadonly(this.options.readonly);
            }
            if (this.options.mode) {
                this.setLanguage(this.options.mode);
            }
            if (this.options.theme) {
                this.setTheme(this.options.theme);
            }
            if (this.options.lineWrapping) {
                this.setLineWrapping(this.options.lineWrapping);
            }
            this.view?.contentDOM.addEventListener('focus', () => {
                this.focusChange.emit(true);
            });
            this.view?.contentDOM.addEventListener('blur', () => {
                this.focusChange.emit(false);
            });
        });
    }
    applyEditorHeight() {
        const editor = this.elementRef.nativeElement.querySelector(".cm-editor");
        if (this.autoMaxHeight) {
            editor.style.maxHeight = `${this.autoMaxHeight}px`;
        }
    }
    setOptionIfChanged(optionName, newValue) {
        if (!this.view) {
            return;
        }
        if (optionName === 'readOnly') {
            this.setReadonly(newValue);
        }
        if (optionName === 'mode') {
            this.setLanguage(newValue);
        }
        if (optionName === 'cursorBlinkRate') {
            this.setCursor(newValue);
        }
        if (optionName === 'theme') {
            this.setTheme(newValue);
        }
        if (optionName === 'lineWrapping') {
            this.setLineWrapping(newValue);
        }
        if (optionName === 'lineNumbers') {
            this.setExtensions(this._getAllExtensions());
        }
    }
    setLineWrapping(value) {
        this._dispatchEffects(this._lineWrappingConf.reconfigure(value ? EditorView.lineWrapping : []));
    }
    setExtensions(value) {
        this._dispatchEffects(StateEffect.reconfigure.of(value));
    }
    setValue(value) {
        this.view?.dispatch({
            changes: { from: 0, to: this.view.state.doc.length, insert: value },
        });
    }
    setReadonly(value) {
        this._dispatchEffects(this._readonlyConf.reconfigure(EditorState.readOnly.of(value)));
    }
    setLanguage(lang) {
        if (!lang) {
            return;
        }
        if (this.languages.length === 0) {
            if (this.view) {
                console.error('No supported languages. Please set the `languages` prop at first.');
            }
            return;
        }
        const langDesc = this._findLanguage(lang || lang.name);
        langDesc?.load().then(lang => {
            this._dispatchEffects(this._languageConf.reconfigure([lang]));
        });
    }
    setCursor(cursorBlinkRate) {
        const cursorStyle = cursorBlinkRate < 0 ? { border: '0px' } : undefined;
        this._dispatchEffects(this._themeConf.reconfigure(EditorView.theme({
            '.cm-cursor': cursorStyle
        })));
    }
    setTheme(value) {
        this._dispatchEffects(this._themeConf.reconfigure(value === 'light' ? [] : value === 'dark' ? oneDark : value === 'material' ? material : value));
    }
    _dispatchEffects(effects) {
        return this.view?.dispatch({ effects });
    }
    _findLanguage(name) {
        for (const lang of this.languages) {
            for (const alias of [lang.name, ...lang.alias]) {
                if (name.toLowerCase() === alias.toLowerCase()) {
                    return lang;
                }
            }
        }
        return null;
    }
    _getAllExtensions() {
        return [
            this._updateListener,
            this._readonlyConf.of([]),
            this._themeConf.of([]),
            this._placeholderConf.of([]),
            this._indentWithTabConf.of([]),
            this._indentUnitConf.of([]),
            this._lineWrappingConf.of([]),
            this._highlightWhitespaceConf.of([]),
            this._languageConf.of([]),
            this.options.lineNumbers ? basicSetup : minimalSetup,
            ...this.extensions
        ];
    }
    ngOnDestroy() {
        if (this._codemirrorContentObserver) {
            this._codemirrorContentObserver.disconnect();
        }
    }
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.0", ngImport: i0, type: CodeMirrorComponent, deps: [{ token: i0.ElementRef }, { token: i0.NgZone }, { token: i0.Renderer2 }, { token: i0.KeyValueDiffers }], target: i0.ɵɵFactoryTarget.Component }); }
    static { this.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "18.2.0", type: CodeMirrorComponent, isStandalone: true, selector: "ng-codemirror, [ngCodeMirror]", inputs: { autoMaxHeight: "autoMaxHeight", options: "options", delayRefreshTime: "delayRefreshTime", extensions: "extensions", languages: "languages" }, outputs: { focusChange: "focusChange" }, host: { properties: { "class.ng-codemirror": "this.codemirrorClassName" } }, providers: [{
                provide: NG_VALUE_ACCESSOR,
                useExisting: forwardRef(() => CodeMirrorComponent),
                multi: true
            }], viewQueries: [{ propertyName: "textAreaRef", first: true, predicate: ["textAreaRef"], descendants: true, read: ElementRef, static: true }], usesOnChanges: true, ngImport: i0, template: `<textarea #textAreaRef style="display:none;"></textarea>`, isInline: true, changeDetection: i0.ChangeDetectionStrategy.OnPush }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.0", ngImport: i0, type: CodeMirrorComponent, decorators: [{
            type: Component,
            args: [{
                    selector: "ng-codemirror, [ngCodeMirror]",
                    template: `<textarea #textAreaRef style="display:none;"></textarea>`,
                    preserveWhitespaces: false,
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    providers: [{
                            provide: NG_VALUE_ACCESSOR,
                            useExisting: forwardRef(() => CodeMirrorComponent),
                            multi: true
                        }],
                    standalone: true
                }]
        }], ctorParameters: () => [{ type: i0.ElementRef }, { type: i0.NgZone }, { type: i0.Renderer2 }, { type: i0.KeyValueDiffers }], propDecorators: { autoMaxHeight: [{
                type: Input
            }], options: [{
                type: Input
            }], delayRefreshTime: [{
                type: Input
            }], extensions: [{
                type: Input
            }], languages: [{
                type: Input
            }], focusChange: [{
                type: Output
            }], textAreaRef: [{
                type: ViewChild,
                args: ["textAreaRef", { read: ElementRef, static: true }]
            }], codemirrorClassName: [{
                type: HostBinding,
                args: ["class.ng-codemirror"]
            }] } });

class CodemirrorModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.0", ngImport: i0, type: CodemirrorModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "18.2.0", ngImport: i0, type: CodemirrorModule, imports: [CommonModule, CodeMirrorComponent], exports: [CodeMirrorComponent] }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "18.2.0", ngImport: i0, type: CodemirrorModule, imports: [CommonModule] }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.0", ngImport: i0, type: CodemirrorModule, decorators: [{
            type: NgModule,
            args: [{
                    imports: [CommonModule, CodeMirrorComponent],
                    exports: [CodeMirrorComponent]
                }]
        }] });

/*
 * Public API Surface of ng-codemirror
 */

/**
 * Generated bundle index. Do not edit.
 */

export { CodeMirrorComponent, CodemirrorModule, External, selectableLanguages };
//# sourceMappingURL=ng-codemirror.mjs.map
